const { getClient, checkParams, getResponse } = require('./utils.js');
var _ = require('underscore');

/*
 Extract the information about the customer-created API keys, the key types and the key usage.
*/
var es_host = process.argv[2];
var es_port = process.argv[3];
var cbd_env = process.argv[4];
var start_time = process.argv[5] || new Date().getTime() - 24 * 60 * 60 * 1000; // Starting 24 hours ago
var end_time = process.argv[6] || new Date().getTime();
var es_user = process.env.ES_USER;
var es_password = process.env.ES_PASSWORD;
var batch = 500;
var max_org_id = 1;

function getQuery(start_org_id, end_org_id, start_time, end_time, is_mcs2) {
    var query = {
      ...(!is_mcs2 && {requestTimeout: 30000}),
      index: 'a_connector',
      body: {
        "size": 10000,
        "query": {
          "bool": {
            "must": [
            {
              "range": {
                "orgId": {
                  "gte": start_org_id,
                  "lt": end_org_id
                }
              }
            }
          ],
          "filter": {
            "bool": {
              "should": [
                {
                  "range": {
                    "createTime": {
                      "gte": start_time,
                      "lt": end_time
                    }
                  }
                },
                {
                  "range": {
                    "lastUpdatedTime": {
                      "gte": start_time,
                      "lt": end_time
                    }
                  }
                },
                {
                  "range": {
                    "stats.lastReportedTime": {
                      "gte": start_time,
                      "lt": end_time
                    }
                  }
                }
              ]
            }
          }
        }
      },
        "_source": {
          "includes": [
            "orgId",
            "connectorType",
            "connectorId",
            "createTime",
            "lastUpdatedTime",
            "name",
            "description",
            "stats.lastReportedTime"
          ]
        }
      }
    };
    return query;
  }

  function processResponse(response, is_mcs2) {
    agg = getResponse(response, is_mcs2);

    if (!_.isUndefined(agg.hits) && !_.isUndefined(agg.hits.hits)) {
      agg.hits.hits.forEach(element => {
        if (!_.isUndefined(element._source)) {
          if (!_.isUndefined(element._source.stats) && !_.isUndefined(element._source.stats.lastReportedTime)) {
            // Flatten the stats structure, extracting the lastReportedTime
            element._source.lastReportedTime = element._source.stats.lastReportedTime;
            delete element._source.stats;
          }
          console.log(JSON.stringify(element._source));
        }
      });
    }
  }

  async function dailyExport(client, is_mcs2) {
    // find max orgId
    await client.search({
        ...(!is_mcs2 && {requestTimeout: 30000}),
        index: 'a_connector',
        body: {
        "query": {
            "query_string": {
            "query": "*"
            }
        },
        "aggregations": {
            "max_orgId": {
            "max": {
                "field": "orgId"
            }
            }
        },
        "size": 0
        }
    }).then(
      function (response) {
        agg = getResponse(response, is_mcs2);

        if (!_.isUndefined(agg.aggregations.max_orgId)) {
          max_org_id = agg.aggregations.max_orgId.value;
        } else {
          console.log("unable to fetch max org_id to iterate from connector");
          process.exit(1);
        }
      }, function (error) {
        console.log(error.message);
        process.exit(1);
      });

    // get org stats in batches
    var from_org_id = 0;
    var maxRetries = 5;
    var retry = 0;
    do {
    to_org_id = from_org_id + batch;
    var queryToExecute = getQuery(from_org_id, to_org_id, start_time, end_time, is_mcs2);
    await client.search(queryToExecute)
        .then(
        function (response) {
            processResponse(response, is_mcs2);
            from_org_id = to_org_id;
            retry = 0;
        }, function (error) {
            retry = retry + 1;
            if (retry > maxRetries) {
            console.log('failed after (' + maxRetries + ') retries. ' + error.message);
            process.exit(1);
            }
        });
    } while (to_org_id < max_org_id);
    process.exit(0);
  }

  (async() => {
    if(es_password && es_user) {
      var is_mcs2 = true;
    } else {
      var is_mcs2 = false;
    }

    checkParams(
      es_host,
      es_port,
      es_user,
      es_password,
      is_mcs2,
      "app-apikeystats.js",
      is_mcs2 ?
      "<es_host> <es_port> <es_user> <es_password> <cbd_env> <optional: start_time> <optional: end_time>" :
      "<es_host> <es_port> <cbd_env> <optional: start_time> <optional: end_time>");


    var client = getClient(es_host, es_port, es_user, es_password, is_mcs2);

    if(!is_mcs2) {
      client.ping({
        requestTimeout: 30000,
      }, function (error) {
        if (error) {
          console.error('The cluster is down!');
          console.error(error);
          process.exit(1);
        }
      });
    }

    await dailyExport(client, is_mcs2);
  })()
