const { getClient, checkParams, getResponse, getIndexName } = require('./utils.js');
var _ = require('underscore');

var es_host = process.argv[2];
var es_port = process.argv[3];
var cbd_env = process.argv[4] || process.env.ENV;
var start_time = process.argv[5] || new Date().getTime() - 24*60*60*1000; // Starting 24 hours ago
var end_time = process.argv[6] || new Date().getTime();
var es_user = process.env.ES_USER;
var es_password = process.env.ES_PASSWORD;
var batch = 500;
var max_org_id = 1;

function getQuery (start_org_id, end_org_id, is_mcs2) {
    var query = {
        ...(!is_mcs2 && {requestTimeout: 120000}),
        index: getIndexName('auditlog', cbd_env, is_mcs2),
        ...(!is_mcs2 && {type: 'log'}),
        body: {
                "size": 0,
                "query": {
                  "bool": {
                    "must": {
                      "query_string": {
                        "query": "*"
                      }
                    },
                    "filter": {
                      "bool": {
                        "must": [
                          {
                            "range": {
                              "orgId": {
                                "gte": start_org_id,
                                "lt": end_org_id
                              }
                            }
                          },
                          {
                            "range": {
                              "createTime": {
                                "gte": start_time,
                                "lt": end_time
                              }
                            }
                          }
                        ]
                      }
                    }
                  }
                },
                "aggregations": {
                  "orgIds": {
                    "terms": {
                      "field": "orgId",
                      "size": 10000
                    },
                    "aggregations": {
                      "users": {
                        "filter": {
                          "term": {
                            "description": "user"
                          }
                        },
                        "aggregations": {
                          "usersCreated": {
                            "filter": {
                              "term": {
                                "description": "created"
                              }
                            }
                          },
                          "usersDeleted": {
                            "filter": {
                              "term": {
                                "description": "deleted"
                              }
                            }
                          }
                        }
                      },
                      "roles": {
                        "filter": {
                          "term": {
                            "description": "role"
                          }
                        },
                        "aggregations": {
                          "rolesCreated": {
                            "filter": {
                              "term": {
                                "description": " created"
                              }
                            }
                          },
                          "rolesDeleted": {
                            "filter": {
                              "term": {
                                "description": "deleted"
                              }
                            }
                          }
                        }
                      },
                      "userLogins": {
                        "filter": {
                          "bool" : {
                            "should" : [
                              {"term" : { "description" : "logged" } },
                              {"term" : { "description" : "login" } }
                            ]
                          }
                        },
                        "aggregations": {
                          "totalLogins": {
                            "filter": {
                              "bool": { "must": { "match": { "description": "successfully" } } }
                            },
                            "aggregations": {
                              "totalUserLogins": {
                                "filter": {
                                  "bool": { "must_not": { "match": { "description": "Connector" } } }
                                },
                                "aggregations": {
                                  "totalUniqueUserLogins": {
                                    "terms": {
                                      "field": "loginName.keyword",
                                      "missing": "N/A",
                                      "size": 500
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      },
                      "alerts": {
                        "filter": {
                          "term": {
                            "description": "alert"
                          }
                        },
                        "aggregations": {
                          "alertsDismissed": {
                            "filter": {
                              "term": {
                                "description": "dismissed"
                              }
                            }
                          },
                          "watchlists": {
                              "filter": {
                                "term": {
                                  "description": "watchlist"
                                }
                              },
                              "aggregations":{
                              "watchlistsCreated": {
                                "filter": {
                                  "term": {
                                    "description": "created"
                                  }
                                }
                              },
                              "watchlistsUsed": {
                                "filter": {
                                  "term": {
                                    "description": "updated"
                                  }
                                }
                              },
                              "watchlistsDeleted": {
                                "filter": {
                                  "term": {
                                    "description": "deleted"
                                  }
                                }
                              }
                            }
                          }
                        }
                      },
                      "hashes": {
                        "filter": {
                          "match_phrase": {
                            "description": "Reputation Override"
                          }
                        },
                        "aggregations": {
                          "hashesDeleted": {
                            "filter": {
                              "term": {
                                "description": "delete"
                              }
                            }
                          },
                          "hashesAllowed": {
                            "filter": {
                              "term": {
                                "description": "WHITE_LIST"
                              }
                            }
                          },
                          "hashesDenied": {
                            "filter": {
                              "term": {
                                "description": "BLACK_LIST"
                              }
                            }
                          }
                        }
                      },
                      "policies": {
                        "filter": {
                          "term": {
                            "description": "policy"
                          }
                        },
                        "aggregations": {
                          "policiesCreated": {
                            "filter": {
                              "term": {
                                "description": "created"
                              }
                            }
                          },
                          "policiesDeleted": {
                            "filter": {
                              "term": {
                                "description": "deleted"
                              }
                            }
                          },
                          "policiesChanged": {
                            "filter": {
                              "term": {
                                "description": "modified"
                              }
                            }
                          }
                        }
                      },
                      "recommendations": {
                        "filter": {
                          "term": {
                            "description": "recommendation"
                          }
                        },
                        "aggregations": {
                          "totalAccepted": {
                            "filter": {
                              "term": {
                                "description": "accepted"
                              }
                            },
                            "aggregations": {
                              "totalAcceptedHash": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "SHA-256 hash"
                                  }
                                }
                              },
                              "totalAcceptedITtool": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "IT tool path"
                                  }
                                }
                              },
                              "totalAcceptedCert": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "certificate publisher"
                                  }
                                }
                              }
                            }
                          },
                          "totalRejected": {
                            "filter": {
                              "term": {
                                "description": "rejected"
                              }
                            },
                            "aggregations": {
                              "totalRejectedHash": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "SHA-256 hash"
                                  }
                                }
                              },
                              "totalRejectedITtool": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "IT tool path"
                                  }
                                }
                              },
                              "totalRejectedCert": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "certificate publisher"
                                  }
                                }
                              }
                            }
                          },
                          "totalReverted": {
                            "filter": {
                              "term": {
                                "description": "reverted"
                              }
                            },
                            "aggregations": {
                              "totalRevertedHash": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "SHA-256 hash"
                                  }
                                }
                              },
                              "totalRevertedITtool": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "IT tool path"
                                  }
                                }
                              },
                              "totalRevertedCert": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "certificate publisher"
                                  }
                                }
                              }
                            }
                          },
                          "totalRestored": {
                            "filter": {
                              "term": {
                                "description": "restored"
                              }
                            },
                            "aggregations": {
                              "totalRestoredHash": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "SHA-256 hash"
                                  }
                                }
                              },
                              "totalRestoredITtool": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "IT tool path"
                                  }
                                }
                              },
                              "totalRestoredCert": {
                                "filter": {
                                  "match_phrase": {
                                    "description": "certificate publisher"
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
        };
        //console.log("query: " + JSON.stringify(query));
        return query;
}

function processResponse(response) {
    if (!_.isUndefined(response.aggregations) && !_.isUndefined(response.aggregations.orgIds) && !_.isUndefined(response.aggregations.orgIds.buckets)) {
        var org_buckets = response.aggregations.orgIds.buckets;
        // console.log("processing stats for (" + org_buckets.length + ") orgs");
        org_buckets.forEach(function (orgIdEntry) {
             var org = {
                "id": orgIdEntry.key,
               // "env": cbd_env,
                "usersCreated": 0, // How many active users until that day/ separate query
                "usersDeleted": 0, //
                "rolesCreated": 0,
                "rolesDeleted": 0,
                "totalLogins": 0,
                "totalUserLogins": 0,
                "totalUniqueUserLogins": 0,
                "alertsDismissed": 0,
                "policiesCreated": 0,
                "policiesChanged": 0,
                "policiesDeleted": 0,
                "hashesDenied": 0,
                "hashesDeleted": 0,
                "hashesAllowed": 0,
                "watchlistCreated": 0,
                "watchlistUsed": 0,
                "watchlistDeleted": 0,
                "totalRecommendationsAccepted": 0,
                "totalRecommendationsAccepted": 0,
                "recommendationsAccepted": [],
                "totalRecommendationsRejected": 0,
                "recommendationsRejected": [],
                "totalRecommendationsReverted": 0,
                "recommendationsReverted": [],
                "totalRecommendationsRestored": 0,
                "recommendationsRestored": [],
             };

             org.usersCreated = orgIdEntry.users.usersCreated.doc_count;
             org.usersDeleted = orgIdEntry.users.usersDeleted.doc_count;
             org.rolesCreated = orgIdEntry.roles.rolesCreated.doc_count;
             org.rolesDeleted = orgIdEntry.roles.rolesDeleted.doc_count;
             org.totalLogins = orgIdEntry.userLogins.totalLogins.doc_count;
             org.totalUserLogins = orgIdEntry.userLogins.totalLogins.totalUserLogins.doc_count;
             org.totalUniqueUserLogins = orgIdEntry.userLogins.totalLogins.totalUserLogins.totalUniqueUserLogins.buckets.length;
             org.alertsDismissed = orgIdEntry.alerts.alertsDismissed.doc_count;
             org.policiesChanged = orgIdEntry.policies.policiesChanged.doc_count;
             org.policiesCreated = orgIdEntry.policies.policiesCreated.doc_count;
             org.hashesDenied = orgIdEntry.hashes.hashesDenied.doc_count;
             org.hashesAllowed = orgIdEntry.hashes.hashesAllowed.doc_count;
             org.hashesDeleted = orgIdEntry.hashes.hashesDeleted.doc_count;
             org.watchlistCreated = orgIdEntry.alerts.watchlists.watchlistsCreated.doc_count;
             org.watchlistUsed = orgIdEntry.alerts.watchlists.watchlistsUsed.doc_count;
             org.watchlistDeleted = orgIdEntry.alerts.watchlists.watchlistsDeleted.doc_count;

             org.totalRecommendationsRejected = orgIdEntry.recommendations.totalRejected.doc_count;
             org.recommendationsRejected.push({"type": "sha256", "value": orgIdEntry.recommendations.totalRejected.totalRejectedHash.doc_count});
             org.recommendationsRejected.push({"type": "itTool", "value": orgIdEntry.recommendations.totalRejected.totalRejectedITtool.doc_count});
             org.recommendationsRejected.push({"type": "cert", "value": orgIdEntry.recommendations.totalRejected.totalRejectedCert.doc_count});

             org.totalRecommendationsAccepted = orgIdEntry.recommendations.totalAccepted.doc_count;
             org.recommendationsAccepted.push({"type": "sha256", "value": orgIdEntry.recommendations.totalAccepted.totalAcceptedHash.doc_count});
             org.recommendationsAccepted.push({"type": "itTool", "value": orgIdEntry.recommendations.totalAccepted.totalAcceptedITtool.doc_count});
             org.recommendationsAccepted.push({"type": "cert", "value": orgIdEntry.recommendations.totalAccepted.totalAcceptedCert.doc_count});

             org.totalRecommendationsRestored = orgIdEntry.recommendations.totalRestored.doc_count;
             org.recommendationsRestored.push({"type": "sha256", "value": orgIdEntry.recommendations.totalRestored.totalRestoredHash.doc_count});
             org.recommendationsRestored.push({"type": "itTool", "value": orgIdEntry.recommendations.totalRestored.totalRestoredITtool.doc_count});
             org.recommendationsRestored.push({"type": "cert", "value": orgIdEntry.recommendations.totalRestored.totalRestoredCert.doc_count});

             org.totalRecommendationsReverted = orgIdEntry.recommendations.totalReverted.doc_count;
             org.recommendationsReverted.push({"type": "sha256", "value": orgIdEntry.recommendations.totalReverted.totalRevertedHash.doc_count});
             org.recommendationsReverted.push({"type": "itTool", "value": orgIdEntry.recommendations.totalReverted.totalRevertedITtool.doc_count});
             org.recommendationsReverted.push({"type": "cert", "value": orgIdEntry.recommendations.totalReverted.totalRevertedCert.doc_count});

             console.log(JSON.stringify(org));
        });
    }
}

async function dailyExport(client, is_mcs2) {
    // find max orgId
    await client.search({
        ...(!is_mcs2 && {requestTimeout: 120000}),
        index: getIndexName('auditlog', cbd_env, is_mcs2),
        ...(!is_mcs2 && {type: 'log'}),
        body: {
            "query": {
                "query_string": {
                    "query": "*"
                }
            },
            "aggregations" : {
                "max_orgId" : {
                    "max" : {
                        "field" : "orgId"
                    }
                }
            },
            "size": 0
        }
    }).then(
        function (response) {
            agg = getResponse(response, is_mcs2);

            if (!_.isUndefined(agg.aggregations.max_orgId)) {
                // console.log("max orgid: " + agg.aggregations.max_orgId.value);
                max_org_id = agg.aggregations.max_orgId.value;
            } else {
                console.log("unable to fetch max org_id to iterate from auditlog");
                process.exit(1);
            }
        }, function (error){
            console.log(error.message);
            process.exit(1);
        });

    // get org stats in batches
    var from_org_id = 0;
    var maxRetries = 5;
    var retry = 0;
    do {
        to_org_id = from_org_id + batch;
        var queryToExecute = getQuery(from_org_id, to_org_id, is_mcs2);
        await client.search(queryToExecute)
                    .then(
                        function (response) {
                            agg = getResponse(response, is_mcs2);
                            processResponse(agg);
                            from_org_id = to_org_id;
                            retry = 0;
                        }, function (error){
                            retry = retry + 1;
                            if (retry > maxRetries) {
                                console.log('failed after ('+ maxRetries + ') retries. ' + error.message);
                                process.exit(1);
                            }
                        });
    } while (to_org_id < max_org_id);
    process.exit(0);
}

(async() => {
  if(es_password && es_user) {
    var is_mcs2 = true;
  } else {
    var is_mcs2 = false;
  }

  checkParams(
    es_host,
    es_port,
    es_user,
    es_password,
    is_mcs2,
    "app-auditlog.js",
    is_mcs2 ? "<es_host> <es_port> <cbd_env> <es_user> <es_password> <optional: start_time> <optional: end_time>" : "<es_host> <es_port> <cbd_env> <optional: start_time> <optional: end_time>");

  var client = getClient(es_host, es_port, es_user, es_password, is_mcs2);

  if(!is_mcs2) {
    client.ping({
      requestTimeout: 30000,
    }, function (error) {
      if (error) {
        console.error('The cluster is down!');
        console.error(error);
        process.exit(1);
      }
    });
  }

  await dailyExport(client, is_mcs2);
})()
