const { getClient, checkParams, getResponse } = require('./utils.js');
var _ = require('underscore');

var es_host = process.argv[2];
var es_port = process.argv[3];
var cbd_env = process.argv[4];
var start_time = process.argv[5] || new Date().getTime() - 24*60*60*1000; // Starting 24 hours ago
var end_time = process.argv[6] || new Date().getTime();
var es_user = process.env.ES_USER;
var es_password = process.env.ES_PASSWORD;
var batch = 500;
var max_org_id = 1;

function getQuery (start_org_id, end_org_id, is_mcs2) {
    var query = {
        ...(!is_mcs2 && {requestTimeout: 30000}),
        index: 'a_devicestats',
        type: 'device',
        body: {
            "query": {
                "bool": {
                    "must": {
                        "query_string": {
                            "query": "AV_ACTIVE AND status:REGISTERED AND lastContact:>" + start_time
                        }
                    },
                    "filter": {
                        "bool": {
                            "must": [
                                {
                                    "exists": {
                                        "field": "avStatus"
                                    }
                                },
                                {
                                    "range": {
                                        "organizationId": {
                                            "gte": + start_org_id,
                                            "lt" : + end_org_id
                                        }
                                    }
                                }
                            ]
                        }
                    }
                }
            },
            "aggregations": {
                "orgs": {
                    "terms": {
                        "field": "organizationName",
                        "size": 2000
                    }
                }
            },
            "size": 0
        }
    };
    //console.log("query: " + JSON.stringify(query));
    return query;
}

function processResponse(response, is_mcs2) {
    // print log from each response
    if (_.isUndefined(response)) {
        return;
    }

    agg = getResponse(response, is_mcs2);

    if (!_.isUndefined(agg.aggregations &&
        !_.isUndefined(agg.aggregations.orgs) &&
        !_.isUndefined(agg.aggregations.orgs.buckets))) {
            agg.aggregations.orgs.buckets.forEach(function (entry) {
            console.log(entry.key + ',' + entry.doc_count);
        });
    }
}

async function dailyExport(client, is_mcs2) {
    // find max orgId
    await client.search({
        ...(!is_mcs2 && {requestTimeout: 30000}),
        index: 'a_devicestats',
        type: 'device',
        body: {
            "query": {
                "query_string": {
                    "query": "*"
                }
            },
            "aggregations" : {
                "max_orgId" : {
                    "max" : {
                        "field" : "organizationId"
                    }
                }
            },
            "size": 0
        }}).then(
            function (response) {
                agg = getResponse(response, is_mcs2);

                if (!_.isUndefined(agg.aggregations.max_orgId)) {
                    // console.log("max orgid: " + agg.aggregations.max_orgId.value);
                    max_org_id = agg.aggregations.max_orgId.value;
                } else {
                    console.log("unable to fetch max org_id to iterate from devicestats");
                    process.exit(1);
                }
            }, function (error){
                console.log(error.message);
                process.exit(1);
            });

    // get org stats in batches
    var from_org_id = 0;
    var maxRetries = 5;
    var retry = 0;
    do {
        to_org_id = from_org_id + batch;
        //console.log("start processing");
        var queryToExecute = getQuery(from_org_id, to_org_id, is_mcs2);
        await client.search(queryToExecute)
                    .then(
                        function (response) {
                            // console.log("processing response orgs (" + from_org_id + " .. " + to_org_id + ")");
                            processResponse(response, is_mcs2);
                            from_org_id = to_org_id;
                            retry = 0;
                        }, function (error){
                            retry = retry + 1;
                            if (retry > maxRetries) {
                                console.log('failed after ('+ maxRetries + ') retries. ' + error.message);
                                process.exit(1);
                            }
                        });
    } while (to_org_id < max_org_id);
    //console.log("done processing");
    process.exit(0);
}

(async() => {
    if(es_port == 443) {
      var is_mcs2 = true;
    } else {
      var is_mcs2 = false;
    }

    checkParams(
        es_host,
        es_port,
        es_user,
        es_password,
        is_mcs2,
        "app-avstats.js",
        is_mcs2 ? "<es_host> <es_port> <es_user> <es_password> <optional: start_time>" : "<es_host> <es_port> <optional: start_time>");

    var client = getClient(es_host, es_port, es_user, es_password, is_mcs2);

    if(!is_mcs2) {
        client.ping({
          requestTimeout: 30000,
        }, function (error) {
          if (error) {
            console.error('The cluster is down!');
            console.error(error);
            process.exit(1);
          }
        });
    }

    await dailyExport(client, is_mcs2);
  })()

