var opensearch = require("@opensearch-project/opensearch");

var elasticsearch = require('elasticsearch');
var _ = require('underscore');

function getClient(es_host, es_port, es_user, es_password, is_mcs2) {
    if(is_mcs2) {
      return new opensearch.Client({
        node: 'https://' + es_host + ':' + es_port,
        auth: {
            username: es_user,
            password: es_password
        }
      });
    } else {
      return new elasticsearch.Client({
        host: es_host + ':' + es_port,
        requestTimeout: 1000 * 60 * 60,
        keepAlive: false,
        log: 'warning'
      });
    }
}

function checkParams(
    es_host,
    es_port,
    es_user,
    es_password,
    is_mcs2,
    script_name,
    params_list) {
    if (is_mcs2) {
        if (_.isUndefined(es_host) || _.isUndefined(es_port) || _.isUndefined(es_user) || _.isUndefined(es_password)) {
        console.log("usage: node " +  script_name + " " + params_list);
        console.log("");
        process.exit();
      }
    } else {
        if (_.isUndefined(es_host) || _.isUndefined(es_port)) {
        console.log("usage: node " + script_name+ " " + params_list);
        console.log("");
        process.exit();
      }
    }
  }

  function getResponse(response, is_mcs2) {
    if (is_mcs2) {
      return response.body;
    } else {
      return response;
    }
}

function getEnvName(envName) {
  const mapping = {
    "local": "local",
    "prd5use1": "prod05",
    "prd2use1": "prod02",
    "prd1use1": "prod01",
    "eap1use1": "eap01",
    "prd1ew2": "prd1ew2",
    "prd1ec1": "prod06",
    "prd1apne1": "prodnrt",
    "prd1apse2": "prodsyd",
    "kcdev01": "dev01",
    "kctest03": "test03",
    "int1usw2": "int1usw2",
    "gint1usw2": "gint1usw2",
    "int2use2": "int2use2"
  };

  if (!mapping.hasOwnProperty(envName)) {
    throw new Error(`EnvName not found for value: ${envName}`);
  }

  return mapping[envName];
}

function getIndexName(indexName, envName, is_mcs2) {
    return is_mcs2? (indexName + '-' + envName): 'a_auditlog';
}

module.exports = { getClient, checkParams, getResponse, getIndexName };
